<?php
require_once SSC_APPLICATION_PATH . '/forms/Abstract/Form.php';

class Ssc_Form_Admin
    extends Ssc_Form_Abstract_Form
{
    protected $_dataKeys = array(
        'keyword_density_min',
        'keyword_density_max',
        'bing_api_key'
    );

    public function load()
    {
        /**
        * @var Ssc_Model_Options
        */
        $model = Ssc_Application::getModel('Options');

        $this->_values = array();
        foreach ($this->_dataKeys as $key) {
            $this->_values[$key] = $model->getValue($key);
        }

        return $this;
    }

    public function save()
    {
        /**
        * @var Ssc_Model_Options
        */
        $model = Ssc_Application::getModel('Options');

        foreach ($this->_dataKeys as $key) {
            $model->setValue($key, $this->getValue($key));
        }

        return $this;
    }


    protected function _validate()
    {
        // keyword_density_min
        $value = $this->getValue('keyword_density_min');
        if ('' == $value) {
            $this->_errors['keyword_density_min'] = 'Keyword Density From: This field is required and cannot be empty';
        } else if ($value < 0 || $value > 100) {
            $this->_errors['keyword_density_min'] = 'Keyword Density From: Value out of range 0..100%';
        }
        $this->_values['keyword_density_min'] = round($value, 1);

        // keyword_density_min
        $value = $this->getValue('keyword_density_max');
        if ('' == $value) {
            $this->_errors['keyword_density_max'] = 'Keyword Density To: This field is required and cannot be empty';
        } else if ($value < 0 || $value > 100) {
            $this->_errors['keyword_density_max'] = 'Keyword Density To: Value out of range 0..100%';
        }
        $this->_values['keyword_density_max'] = round($value, 1);

        // Compare min and max values
        if ($this->_values['keyword_density_max'] < $this->_values['keyword_density_min']) {
            $this->_errors['keyword_density'] = 'Keyword Density: lower value have to equal or less than upper value';
        }

        // bing_api_key
        $value = $this->getValue('bing_api_key');
        $this->_values['bing_api_key'] = trim(str_replace(array("\r", "\n"), '', $value));
    }
}