<?php
require_once SSC_APPLICATION_PATH . '/services/SeoScore/Factor/Abstract/Bool.php';

class Ssc_Service_SeoScore_Factor_KeywordDensityLimits
    extends Ssc_Service_SeoScore_Factor_Abstract_Bool
{
    protected $_minLimit;
    protected $_maxLimit;

    public function init()
    {
        parent::init();

        $options = Ssc_Application::getModel('Options');
        $this->_minLimit = $options->getValue('keyword_density_min');
        $this->_maxLimit = $options->getValue('keyword_density_max');
    }

    public function analyze($service)
    {
        $density = $service->analyzeHtmlWords('keyword_density_percent');

        $this->setValue($density >= $this->_minLimit
            && $density <= $this->_maxLimit);
        $this->setExtraData(array(
            'min' => $this->_minLimit,
            'max' => $this->_maxLimit,
        ));
    }

    public function getLabel()
    {
        $min = $this->_minLimit;
        $max = $this->_maxLimit;

        $needUpdate = FALSE;
        if ($this->_extraData) {
            $value = $this->getExtraData('min');
            if (!is_null($value) && $value != $this->_minLimit) {
                $min = $value;
                $needUpdate = TRUE;
            }
            $value = $this->getExtraData('max');
            if (!is_null($value) && $value != $this->_maxLimit) {
                $max = $value;
                $needUpdate = TRUE;
            }
        }

        $append = $needUpdate? ' [needs update]' : '';

        return sprintf('Content has %01.1f-%01.1f%% keyword density%s.',
            $min, $max, $append);
    }
}
