<?php

/*
Plugin Name: SEO Scorer With Site
Plugin URI: http://elitewpplugins.com/seo-scorer-with-site/
Description: Analyzes SEO factors of your post and pages.
Author: Test User
Version: 1.3.9
Author URI: 
Tested up to: 3.7
*/

/**
* Version history:
* 1.3.9
*   force usage of PluginUpdateChecker v1.3
* 1.3.8
*   new update logic for less server load
* 1.3.7
*   new update URL
* 1.3.5, 1.3.6
*   fixed 7EJ-SX1-1RPA: bug in pocessing of Unicode post URLs
* 1.3.4
*   added: embedded auto-update
* 1.3.3
*   fixed 7EJ-SX1-1RPA: non-latin words were not counted
* 1.3.2
*  small fix
* 1.3.1
*   user fiendly error message for 403 error
* 1.3
*   users now use their own Bing API keys
* 1.2.1
*   added: filtering of scripts and styles prior to strip_tags
* 1.2
*   related terms now use new Bing API
* 1.1.1:
*   added: force related keywords request language based on WP language
* 1.1:
*   added: workaround for blocked loopback connections
* 1.0.11:
*   fixed: potential crash with array-style cookies
* 1.0.10:
*   fixed: getting post preview URL
*   fixed: potential session restart warning
* 1.0.9
*   added: auto-update support
* 1.0.8
*   fixed: problem with WP < 3.1
* 1.0.7
*   fixed: AJAX animation not hidden in old WP versions
* 1.0.6
*   added: instructions and external links
* 1.0.5
*   content factors now analyze the_content function output
* 1.0.4
*   fixed: redirect problems on getting post HTML
* 1.0.3
*   fixed: moving a post to trash was processed as regular post saving
* 1.0.2
*   modified: image alt factor now analyzes whole page HTML
* 1.0.1
*   fixed: Encoding issues
*/

if (!class_exists('Ssc_WpPlugin')) {
    class Ssc_WpPlugin
    {
    //------------------------------------------------------------------------------
    // Static members
    //------------------------------------------------------------------------------
        /**
        * Version tag. Used for CSS & JS versioning.
        */
        const VERSION = '01.03.09';

        /**
        * Unique slug
        */
        const SLUG = 'seo-scorer-with-site';

        /**
        * Unique prefix
        */
        const PREFIX = 'ssc_';

        /**
        * User-friendly title
        */
        const TITLE = 'SEO Scorer With Site';

        /**
        * Menu item text
        */
        const MENU_LABEL = 'SEO Scorer With Site';

        /**
        * Required capability to access plug-in pages
        */
        const CAPABILITY = 'publish_posts';

        static protected $_instance;

        static protected $_pluginBaseUrl;
        static protected $_pluginPath;

        const SUPPORT_UPDATES = TRUE;
        const UPDATE_CHECK_UNTERVAL = 12;
        static protected $_updater;
        static protected $_updaterCronHook;

        /**
        * Registers plug-in module
        */
        static public function init()
        {
            $class = get_class();

            if (self::SUPPORT_UPDATES) {
                // Auto update
                require dirname(__FILE__) . '/plugin-updates/plugin-update-checker.php';
                self::$_updater = new PluginUpdateChecker_1_3(
                    'http://wppluginupdate.com/r/35/info.json',
                    __FILE__,
                    'seo-scorer-with-site',
                    0
                );
                self::$_updaterCronHook = 'auto_check_updates-' . self::SLUG;
                add_action(self::$_updaterCronHook, array(self::$_updater, 'checkForUpdates'));

                add_action('admin_init', array($class, 'autoCheckUpdates'));
            }

            self::$_pluginBaseUrl = trailingslashit(get_bloginfo('wpurl')) . PLUGINDIR
                . '/' . dirname(plugin_basename(__FILE__));

            self::$_pluginPath = dirname(__FILE__);

            register_activation_hook(__FILE__, array($class, 'activate'));
            register_deactivation_hook(__FILE__, array($class, 'deactivate'));

            // Admin actions
            if (is_admin()) {
                add_action('admin_menu', array($class, 'adminMenu'));
                add_action('wp_ajax_ssc', array($class, 'adminAjax'));
                add_action('admin_enqueue_scripts', array($class, 'adminEnqueueScripts'));
                add_action('save_post', array($class, 'savePost'), 10, 2);
                foreach (self::getPostTypes() as $type) {
                    add_filter("manage_{$type}_posts_columns", array($class, 'managePostsColumns'));
                    add_action("manage_{$type}_posts_custom_column" , array($class, 'managePostsCustomColumn'));
                }
            }

            self::addLibraryIncludePath();

            // Disable admin bar on retrieving post HTML
            if (!is_admin() && isset($_GET['p']) && isset($_GET['preview'])
                && isset($_GET['admin_bar']) && 'true' == $_GET['preview']
                && 'false' == $_GET['admin_bar']
            ) {
                if (function_exists('show_admin_bar')) {
                    show_admin_bar(FALSE);
                }
            }
        }

        /**
        * Plug-in activate hook
        */
        static public function activate()
        {

        }

        /**
        * Plug-in deactivation hook
        */
        static public function deactivate()
        {

        }

        static public function addLibraryIncludePath()
        {
        }

        static public function autoCheckUpdates()
        {
            $updater = self::$_updater;
            $state = $updater->getUpdateState();

            $shouldCheck = (empty($state)
                || !isset($state->lastCheck)
                || ((time() - $state->lastCheck) >= (self::UPDATE_CHECK_UNTERVAL * 3600)))
                && !wp_next_scheduled(self::$_updaterCronHook);

            if ($shouldCheck) {
                wp_schedule_single_event(time(), self::$_updaterCronHook);
                spawn_cron();
            }
        }

        /**
        * Hooks
        */
        static public function adminMenu()
        {
            self::getInstance()
                ->_registerAdminPages()
                ->_registerMetaBoxes();
        }

        static public function adminAjax()
        {
            self::getInstance()->dispatchAjax();
        }

        static public function savePost($postId, $post)
        {
            // Ignore new posts, autosave and revisions:
            if (wp_is_post_autosave($postId)
                || wp_is_post_revision($postId)
                || 'auto-draft' == $post->post_status
                || 'trash' == $post->post_status
                || (isset($_POST['action']) && 'autosave' == $_POST['action'])
                || 'nav_menu_item' == $post->post_type
            ) {
                return;
            }

            self::getInstance()->dispatch__metabox__handle_save_post($postId, $post);
        }

        static public function adminEnqueueScripts($hook)
        {
            switch ($hook) {
                // Admin panel
                case 'toplevel_page_' . self::SLUG:
                    wp_enqueue_style('ssc_admin',
                        self::$_pluginBaseUrl . '/css/admin.css', array(),
                        self::VERSION);
                    break;

                // Post editor
                case 'post.php':
                case 'post-new.php':
                    wp_enqueue_style('ssc_metabox',
                        self::$_pluginBaseUrl . '/css/metabox.css', array(),
                        self::VERSION);
                    wp_enqueue_script('ssc_metabox',
                        self::$_pluginBaseUrl . '/js/metabox.js', array('jquery'),
                        self::VERSION);
                    break;

                // Do not load common stuff for other pages
                default:
                    return;
            }

            // Common stuff
            wp_enqueue_style('ssc_help',
                self::$_pluginBaseUrl . '/css/help.css', array(),
                self::VERSION);
            wp_enqueue_script('ssc_help',
                self::$_pluginBaseUrl . '/js/help.js', array('jquery'),
                self::VERSION);
        }

        static public function managePostsColumns($columns)
        {
            $columns[self::PREFIX . 'score'] = 'On-Page SEO Score';
            $columns[self::PREFIX . 'primary'] = 'Primary Keyword';

            return $columns;
        }

        static public function managePostsCustomColumn($name)
        {
            $prefixLen = strlen(Ssc_WpPlugin::PREFIX);
            if (Ssc_WpPlugin::PREFIX != substr($name, 0, $prefixLen)) {
                return;
            }

            $name = substr($name, $prefixLen);

            return self::getInstance()->dispatch__post_columns__value($name);
        }

        /**
        * Misc Helpers
        */

        /**
        * Returns array of available post types
        */
        static protected function getPostTypes()
        {
            return function_exists('get_post_types')
                ? get_post_types('','names')
                : array('post', 'page');
        }

        /**
        * Singleton pattern
        *
        * @return Ssc_WpPlugin Singleton instance
        */
        static public function getInstance()
        {
            if (!self::$_instance) {
                self::$_instance = new self;
            }

            return self::$_instance;
        }

        /**
        * Static accessors
        */

        static public function getPluginBaseUrl()
        {
            return self::$_pluginBaseUrl;
        }

        static public function getPluginPath()
        {
            return self::$_pluginPath;
        }

    //------------------------------------------------------------------------------
    // Object members
    //------------------------------------------------------------------------------
        /**
        * Register plug-in admin pages and menus
        */
        protected function _registerAdminPages()
        {
            /*add_object_page(self::TITLE, self::MENU_LABEL,
                self::CAPABILITY, self::SLUG,
                array($this, 'dispatch__admin__index'),
                self::$_pluginBaseUrl . '/favicon.ico'
            );
            */

            add_menu_page(self::TITLE, self::MENU_LABEL,
                self::CAPABILITY, self::SLUG,
                array($this, 'dispatchAdmin'),
                self::$_pluginBaseUrl . '/images/icon.png'
            );

            add_submenu_page(self::SLUG, self::TITLE,
                'Settings', self::CAPABILITY,
                self::SLUG,
                array($this, 'dispatchAdmin')
            );

            return $this;
        }

        /**
        * Register plug-in admin metaboxes
        */
        protected function _registerMetaBoxes()
        {
            foreach(self::getPostTypes() as $type) {
                add_meta_box('ssc_main', self::TITLE, array($this, 'dispatch__metabox__index'), $type, 'side', 'high' );
            }
        }

        /**
        * Intercepts WP requests
        */
        public function __call($method, $arguments)
        {
            if ('dispatch__' == substr($method, 0, 10)) {
                $parts = explode('__', $method);
                $controller = @$parts[1];
                $action = @$parts[2];

                $controller = $this->_prepareMvcName($controller);
                $action = $this->_prepareMvcName($action);

                return $this->dispatch($controller, $action, $arguments);
            }
            else {
                throw new ErrorException("Method {$method} not found.");
            }
        }

        /**
        * Dispatches user request to WP Module
        *
        * @param string $controller Controller name
        * @param string $action Action name
        */
        public function dispatch($controller='index', $action='index',
            $arguments = NULL)
        {
            self::addLibraryIncludePath();

            require_once(self::$_pluginPath . '/application/Application.php');
            return Ssc_Application::dispatch($controller, $action, $arguments);
        }

        public function dispatchAdmin()
        {
            $controller = @$_GET['controller'];
            if (empty($controller)) $controller = 'admin';
            $action = @$_GET['action'];
            if (empty($action)) $action = 'index';

            $this->dispatch($controller, $action);
        }

        public function dispatchAjax()
        {
            $controller = @$_POST['controller'];
            if (empty($controller)) $controller = 'index';
            $action = @$_POST['controller_action'];
            if (empty($action)) $action = 'index';

            $this->dispatch($controller, $action);
            exit;
        }

        protected function _prepareMvcName($name)
        {
            return str_replace('_', '-', $name);
        }
    }

    //Register plug-in:
    Ssc_WpPlugin::init();
} else {
    deactivate_plugins(plugin_basename(__FILE__));
    trigger_error('This plugin cannot be installed together with '
        . Ssc_WpPlugin::TITLE, E_USER_ERROR);
    return;
}